@@ -1,6 +1,6 @@
 use std::sync::Arc;
 
-use robespierre_models::channel::Message;
+use robespierre_models::channels::Message;
 
 pub mod standard;
 
@@ -9,4 +9,4 @@ pub trait Framework: Send + Sync + 'static {
     type Context: 'static;
 
     async fn handle(&self, ctx: Self::Context, message: &Arc<Message>);
-}
\ No newline at end of file
+}
