@@ -1,8 +1,8 @@
 pub extern crate async_std;
 pub extern crate typemap;
 
-pub extern crate robespierre_models;
 pub extern crate robespierre_http;
+pub extern crate robespierre_models;
 
 #[cfg(feature = "cache")]
 pub extern crate robespierre_cache;
@@ -14,6 +14,7 @@ use std::sync::Arc;
 
 #[cfg(feature = "framework")]
 use framework::Framework;
+use model::ServerIdExt;
 #[cfg(feature = "cache")]
 use robespierre_cache::{Cache, CacheConfig, CommitToCache, HasCache};
 #[cfg(feature = "events")]
@@ -22,11 +23,11 @@ use robespierre_events::{
 };
 use robespierre_http::{Http, HttpAuthentication, HttpError};
 use robespierre_models::{
-    channel::{Channel, ChannelField, Message, PartialChannel, PartialMessage},
+    channels::{Channel, ChannelField, Message, PartialChannel, PartialMessage},
     events::{ReadyEvent, ServerToClientEvent},
     id::{ChannelId, MemberId, MessageId, RoleId, ServerId, UserId},
-    server::{MemberField, PartialMember, PartialRole, PartialServer, RoleField, ServerField},
-    user::{PartialUser, RelationshipStatus, UserField},
+    servers::{MemberField, PartialMember, PartialRole, PartialServer, RoleField, ServerField},
+    users::{RelationshipStatus, UserField, UserPatch},
 };
 
 pub use async_trait::async_trait;
@@ -133,7 +134,7 @@ pub trait EventHandler: Send + Sync {
         &self,
         ctx: Context,
         id: UserId,
-        modifications: PartialUser,
+        modifications: UserPatch,
         remove: Option<UserField>,
     ) {
     }
@@ -149,30 +150,34 @@ pub trait EventHandler: Send + Sync {
 }
 
 /// Wraps an event handler, updating the cache and then forwarding the events
-/// to the wrapped [`EventHandler`]
+/// to the inner [`EventHandler`]
 #[cfg(all(feature = "events", feature = "cache"))]
 #[derive(Clone)]
-pub struct CacheWrap<Ctx: HasCache + Clone + 'static, T: RawEventHandler<Context = Ctx>>(T);
+pub struct CacheWrap<Inner>(Inner)
+where
+    Inner: RawEventHandler,
+    Inner::Context: HasCache + Clone + 'static;
 
 #[cfg(all(feature = "events", feature = "cache"))]
-impl<Ctx, T> CacheWrap<Ctx, T>
+impl<Inner> CacheWrap<Inner>
 where
-    Ctx: HasCache + Clone + 'static,
-    T: RawEventHandler<Context = Ctx>,
+    Inner: RawEventHandler,
+    Inner::Context: HasCache + Clone + 'static,
 {
     /// Creates a new [`CacheWrap`]
-    pub fn new(wrapped: T) -> Self {
-        Self(wrapped)
+    pub fn new(inner: Inner) -> Self {
+        Self(inner)
     }
 }
 
 #[cfg(all(feature = "events", feature = "cache"))]
 #[async_trait::async_trait]
-impl<T, Ctx: HasCache + Clone + 'static> RawEventHandler for CacheWrap<Ctx, T>
+impl<Inner> RawEventHandler for CacheWrap<Inner>
 where
-    T: RawEventHandler<Context = Ctx>,
+    Inner: RawEventHandler,
+    Inner::Context: HasCache + Clone + 'static,
 {
-    type Context = Ctx;
+    type Context = Inner::Context;
 
     async fn handle(self, ctx: Self::Context, event: ServerToClientEvent) {
         event.commit_to_cache_ref(&ctx).await;
@@ -185,22 +190,22 @@ where
 #[derive(Clone)]
 pub struct FrameworkWrap<
     FrameworkContext: From<Context> + Send + Sync + 'static,
-    T: EventHandler + Clone + 'static,
+    Inner: EventHandler + Clone + 'static,
 > {
     fw: Arc<RwLock<Box<dyn Framework<Context = FrameworkContext> + 'static>>>,
-    handler: T,
+    inner: Inner,
 }
 
 #[cfg(all(feature = "events", feature = "framework"))]
 impl<
         FrameworkContext: From<Context> + Send + Sync + 'static,
-        T: EventHandler + Clone + 'static,
-    > FrameworkWrap<FrameworkContext, T>
+        Inner: EventHandler + Clone + 'static,
+    > FrameworkWrap<FrameworkContext, Inner>
 {
-    pub fn new<Fw: Framework<Context = FrameworkContext> + 'static>(fw: Fw, handler: T) -> Self {
+    pub fn new<Fw: Framework<Context = FrameworkContext> + 'static>(fw: Fw, inner: Inner) -> Self {
         Self {
             fw: Arc::new(RwLock::new(Box::new(fw))),
-            handler,
+            inner,
         }
     }
 }
@@ -209,11 +214,11 @@ impl<
 #[async_trait::async_trait]
 impl<
         FrameworkContext: From<Context> + Send + Sync + 'static,
-        T: EventHandler + Clone + 'static,
-    > EventHandler for FrameworkWrap<FrameworkContext, T>
+        Inner: EventHandler + Clone + 'static,
+    > EventHandler for FrameworkWrap<FrameworkContext, Inner>
 {
     async fn on_ready(&self, ctx: Context, ready: ReadyEvent) {
-        self.handler.on_ready(ctx, ready).await
+        self.inner.on_ready(ctx, ready).await
     }
 
     async fn on_message(&self, ctx: Context, message: Message) {
@@ -224,7 +229,7 @@ impl<
             .handle(FrameworkContext::from(ctx.clone()), &message)
             .await;
         let message = Arc::try_unwrap(message).expect("Do not store `Arc<Message>`s while handling them in the framework. Instead, clone the inner `Message`s.");
-        self.handler.on_message(ctx, message).await
+        self.inner.on_message(ctx, message).await
     }
 
     async fn on_message_update(
@@ -234,19 +239,19 @@ impl<
         message: MessageId,
         modifications: PartialMessage,
     ) {
-        self.handler
+        self.inner
             .on_message_update(ctx, channel, message, modifications)
             .await
     }
 
     async fn on_message_delete(&self, ctx: Context, channel_id: ChannelId, message_id: MessageId) {
-        self.handler
+        self.inner
             .on_message_delete(ctx, channel_id, message_id)
             .await
     }
 
     async fn on_channel_create(&self, ctx: Context, channel: Channel) {
-        self.handler.on_channel_create(ctx, channel).await
+        self.inner.on_channel_create(ctx, channel).await
     }
 
     async fn on_channel_update(
@@ -256,29 +261,29 @@ impl<
         modifications: PartialChannel,
         remove: Option<ChannelField>,
     ) {
-        self.handler
+        self.inner
             .on_channel_update(ctx, channel_id, modifications, remove)
             .await
     }
 
     async fn on_channel_delete(&self, ctx: Context, channel_id: ChannelId) {
-        self.handler.on_channel_delete(ctx, channel_id).await
+        self.inner.on_channel_delete(ctx, channel_id).await
     }
 
     async fn on_group_join(&self, ctx: Context, id: ChannelId, user: UserId) {
-        self.handler.on_group_join(ctx, id, user).await
+        self.inner.on_group_join(ctx, id, user).await
     }
 
     async fn on_group_leave(&self, ctx: Context, id: ChannelId, user: UserId) {
-        self.handler.on_group_leave(ctx, id, user).await
+        self.inner.on_group_leave(ctx, id, user).await
     }
 
     async fn on_start_typing(&self, ctx: Context, channel: ChannelId, user: UserId) {
-        self.handler.on_start_typing(ctx, channel, user).await
+        self.inner.on_start_typing(ctx, channel, user).await
     }
 
     async fn on_stop_typing(&self, ctx: Context, channel: ChannelId, user: UserId) {
-        self.handler.on_stop_typing(ctx, channel, user).await
+        self.inner.on_stop_typing(ctx, channel, user).await
     }
 
     async fn on_server_update(
@@ -288,17 +293,17 @@ impl<
         modifications: PartialServer,
         remove: Option<ServerField>,
     ) {
-        self.handler
+        self.inner
             .on_server_update(ctx, server, modifications, remove)
             .await
     }
 
     async fn on_server_delete(&self, ctx: Context, server: ServerId) {
-        self.handler.on_server_delete(ctx, server).await
+        self.inner.on_server_delete(ctx, server).await
     }
 
     async fn on_server_member_join(&self, ctx: Context, server: ServerId, user: UserId) {
-        self.handler.on_server_member_join(ctx, server, user).await
+        self.inner.on_server_member_join(ctx, server, user).await
     }
 
     async fn on_server_member_update(
@@ -308,13 +313,13 @@ impl<
         modifications: PartialMember,
         remove: Option<MemberField>,
     ) {
-        self.handler
+        self.inner
             .on_server_member_update(ctx, member, modifications, remove)
             .await
     }
 
     async fn on_server_member_leave(&self, ctx: Context, server: ServerId, user: UserId) {
-        self.handler.on_server_member_leave(ctx, server, user).await
+        self.inner.on_server_member_leave(ctx, server, user).await
     }
 
     async fn on_server_role_update(
@@ -325,23 +330,23 @@ impl<
         modifications: PartialRole,
         remove: Option<RoleField>,
     ) {
-        self.handler
+        self.inner
             .on_server_role_update(ctx, server, role, modifications, remove)
             .await
     }
 
     async fn on_server_role_delete(&self, ctx: Context, server: ServerId, role: RoleId) {
-        self.handler.on_server_role_delete(ctx, server, role).await
+        self.inner.on_server_role_delete(ctx, server, role).await
     }
 
     async fn on_user_update(
         &self,
         ctx: Context,
         id: UserId,
-        modifications: PartialUser,
+        modifications: UserPatch,
         remove: Option<UserField>,
     ) {
-        self.handler
+        self.inner
             .on_user_update(ctx, id, modifications, remove)
             .await
     }
@@ -353,22 +358,81 @@ impl<
         other_id: UserId,
         status: RelationshipStatus,
     ) {
-        self.handler
+        self.inner
             .on_user_relationship_update(ctx, self_id, other_id, status)
             .await
     }
 }
 
+/// "Maintains" the list of servers in the cache, keeping it the
+/// same as the list of servers the bot is in, by listening
+/// to the `ServerMember{Join,Leave}` events with the
+/// user id of the bot, which should be passed in [`Self::new`]
+#[cfg(all(feature = "events", feature = "cache"))]
+#[derive(Clone)]
+pub struct CacheServersMaintainer<Inner>
+where
+    Inner: RawEventHandler + Clone,
+    Inner::Context: CacheHttp,
+{
+    user_id: UserId,
+    inner: Inner,
+}
+
+#[cfg(all(feature = "events", feature = "cache"))]
+impl<Inner> CacheServersMaintainer<Inner>
+where
+    Inner: RawEventHandler + Clone,
+    Inner::Context: CacheHttp,
+{
+    /// Creates a new [`CacheServersMaintainer`].
+    ///
+    /// `user_id` should be the user id of the bot.
+    pub fn new(user_id: UserId, inner: Inner) -> Self {
+        Self { user_id, inner }
+    }
+}
+
+#[cfg(all(feature = "events", feature = "cache"))]
+#[async_trait::async_trait]
+impl<Inner> RawEventHandler for CacheServersMaintainer<Inner>
+where
+    Inner: RawEventHandler + Clone,
+    Inner::Context: CacheHttp,
+{
+    type Context = Inner::Context;
+
+    async fn handle(self, ctx: Self::Context, event: ServerToClientEvent) {
+        if let Some(cache) = ctx.cache() {
+            match &event {
+                ServerToClientEvent::ServerMemberJoin { id, user } => {
+                    if *user == self.user_id {
+                        let _ = id.server(&ctx).await; // will fetch server and store to cache
+                    }
+                }
+                ServerToClientEvent::ServerMemberLeave { id, user } => {
+                    if *user == self.user_id {
+                        cache.delete_server(*id).await;
+                    }
+                }
+                _ => {}
+            }
+        }
+
+        self.inner.handle(ctx, event).await
+    }
+}
+
 /// An object that can be passed to [`robespierre_events::Connection::run`], and
 /// distinguishes between the events and calls the relevant handler.
 #[cfg(feature = "events")]
 #[derive(Clone)]
-pub struct EventHandlerWrap<T: EventHandler + Clone + 'static>(T);
+pub struct EventHandlerWrap<Inner: EventHandler + Clone + 'static>(Inner);
 
 #[cfg(feature = "events")]
-impl<T: EventHandler + Clone + 'static> EventHandlerWrap<T> {
-    pub fn new(handler: T) -> Self {
-        Self(handler)
+impl<Inner: EventHandler + Clone + 'static> EventHandlerWrap<Inner> {
+    pub fn new(inner: Inner) -> Self {
+        Self(inner)
     }
 }
 
@@ -384,8 +448,8 @@ where
         match event {
             ServerToClientEvent::Error { error } => tracing::error!("Error: {}", error),
             ServerToClientEvent::Authenticated => {}
-            ServerToClientEvent::Pong { time } => {
-                tracing::debug!("Got a pong from the server, time: {}", time)
+            ServerToClientEvent::Pong { data } => {
+                tracing::debug!("Got a pong from the server, time: {}", data)
             }
             ServerToClientEvent::Ready { event } => self.0.on_ready(ctx, event).await,
             ServerToClientEvent::Message { message } => self.0.on_message(ctx, message).await,
@@ -458,13 +522,8 @@ where
 }
 
 pub enum Authentication {
-    Bot {
-        token: String,
-    },
-    User {
-        user_id: UserId,
-        session_token: String,
-    },
+    Bot { token: String },
+    User { session_token: String },
 }
 
 impl Authentication {
@@ -474,9 +533,8 @@ impl Authentication {
         }
     }
 
-    pub fn user(user_id: UserId, session_token: impl Into<String>) -> Self {
+    pub fn user(session_token: impl Into<String>) -> Self {
         Self::User {
-            user_id,
             session_token: session_token.into(),
         }
     }
@@ -489,11 +547,7 @@ impl<'a> From<&'a Authentication> for robespierre_events::Authentication<'a> {
             Authentication::Bot { token } => Self::Bot {
                 token: token.as_str(),
             },
-            Authentication::User {
-                user_id,
-                session_token,
-            } => Self::User {
-                user_id: *user_id,
+            Authentication::User { session_token } => Self::User {
                 session_token: session_token.as_str(),
             },
         }
@@ -506,11 +560,7 @@ impl<'a> From<&'a Authentication> for HttpAuthentication<'a> {
             Authentication::Bot { token } => Self::BotToken {
                 token: token.as_str(),
             },
-            Authentication::User {
-                user_id,
-                session_token,
-            } => Self::UserSession {
-                user_id: *user_id,
+            Authentication::User { session_token } => Self::UserSession {
                 session_token: session_token.as_str(),
             },
         }
@@ -595,7 +645,7 @@ impl robespierre_events::Context for Context {
 #[cfg(feature = "cache")]
 impl HasCache for Context {
     fn get_cache(&self) -> Option<&Cache> {
-        self.cache.as_ref().map(|it| &**it)
+        self.cache.as_deref()
     }
 }
 
@@ -643,3 +693,6 @@ impl<T: HasHttp> CacheHttp for T {
         self.get_http()
     }
 }
+
+#[cfg(doctest)]
+mod booktests;
